@@ -19,6 +19,7 @@
 #include "gdcmDicts.h"
 #include "gdcmEvent.h"
 #include "gdcmGlobal.h"
+#include "gdcmMEC_MR3.h"
 
 #include "gdcmext/csa.h"
 #include "gdcmext/mec_mr3.h"
@@ -638,9 +639,12 @@ struct Cleaner::impl {
     static const PrivateTag &csa1 = CSAHeader::GetCSAImageHeaderInfoTag();
     static const PrivateTag &csa2 = CSAHeader::GetCSASeriesHeaderInfoTag();
     const PrivateTag mec_mr3(0x700d, 0x08, "TOSHIBA_MEC_MR3");
-    const PrivateTag pmtf1(0x0029, 0x90, "PMTF INFORMATION DATA");
-    const PrivateTag pmtf2(0x0029, 0x90, "TOSHIBA_MEC_MR3");
-    if (pt == csa1 || pt == csa2 || pt == mec_mr3 || pt == pmtf1 || pt == pmtf2) {
+    static const PrivateTag &pmtf1 = gdcm::MEC_MR3::GetPMTFInformationDataTag();
+    static const PrivateTag &pmtf2 = gdcm::MEC_MR3::GetToshibaMECMR3Tag();
+    static const PrivateTag &pmtf3 = gdcm::MEC_MR3::GetCanonMECMR3Tag();
+
+    if (pt == csa1 || pt == csa2 || pt == mec_mr3 || pt == pmtf1 ||
+        pt == pmtf2 || pt == pmtf3) {
       scrub_privatetags.insert(pt);
       return true;
     }
@@ -753,11 +757,23 @@ static bool CleanCSA(DataSet &ds, const DataElement &de) {
   if (bv->GetLength() >= 10 && memcmp(bv->GetPointer(), pds_com, 10) == 0) {
     return true;
   }
+  static const char psd_ocm[] = "p<sd<>oc>m";
+  // byte-swap PET_REPLAY_PARAM case:
+  if (bv->GetLength() >= 10 && memcmp(bv->GetPointer(), psd_ocm, 10) == 0) {
+    return true;
+  }
+  // ANGIOHEAD case. This is a DICOM Explicit with a odd ending:
   static const char end[] = "END!      ";
   if (bv->GetLength() >= 10 &&
       memcmp(bv->GetPointer() + bv->GetLength() - 10, end, 10) == 0) {
     return true;
   }
+  // byte-swapped ANGIOHEAD
+  static const char ned[] = "NE!D      ";
+  if (bv->GetLength() >= 10 &&
+      memcmp(bv->GetPointer() + bv->GetLength() - 10, ned, 10) == 0) {
+    return true;
+  }
   const bool zero = isAllZero(bv->GetPointer(), bv->GetLength());
   if (zero) return true;
 
@@ -783,6 +799,17 @@ static bool CleanMEC_MR3(DataSet &ds, const DataElement &de) {
   clean.SetVR(de.GetVR());
   std::vector<char> v;
   v.resize(bv->GetLength());
+  {
+    // check a pseudo magic value here:
+    uint32_t magic = 0xffff;
+    if (bv->GetLength() > 4) {
+      memcpy(&magic, bv->GetPointer(), sizeof magic);
+    }
+    if (magic > 512) {
+      gdcmDebugMacro("Cannot handle MEC_MR3");
+      return true;
+    }
+  }
   if (mec_mr3_memcpy(&v[0], bv->GetPointer(), bv->GetLength())) {
     clean.SetByteValue(&v[0], v.size());
     ds.Replace(clean);
@@ -799,7 +826,9 @@ static bool CleanPMTF(DataSet &ds, const DataElement &de) {
   const char *input = bv->GetPointer();
   const size_t len = bv->GetLength();
 
-  bool success = true;
+  gdcm::Cleaner cleaner;
+  gdcm::File &file = cleaner.GetFile();
+  gdcm::DataSet &revds = file.GetDataSet();
   try {
     std::istringstream is;
     {
@@ -811,13 +840,16 @@ static bool CleanPMTF(DataSet &ds, const DataElement &de) {
 
     // FIXME gdcm::Cleaner will by default change defined length SQ into undef
     // length...there is a risk of incompatibily with vendor
-    gdcm::Cleaner cleaner;
-    gdcm::File &file = cleaner.GetFile();
     gdcm::FileMetaInformation &fmi = file.GetHeader();
     fmi.SetDataSetTransferSyntax(gdcm::TransferSyntax::ExplicitVRLittleEndian);
-    gdcm::DataSet &revds = file.GetDataSet();
     revds.Read<gdcm::ExplicitDataElement, gdcm::SwapperNoOp>(is);
+  } catch (...) {
+    gdcmDebugMacro("Unhanded file format");
+    return true;
+  }
 
+  bool success = true;
+  try {
     gdcm::VR vr = VR::PN;
     cleaner.Empty(vr);
     if (!cleaner.Clean()) {
@@ -1036,8 +1068,10 @@ bool Cleaner::impl::ProcessDataSet(Subject &subject, File &file, DataSet &ds,
       static const PrivateTag &csa1 = CSAHeader::GetCSAImageHeaderInfoTag();
       static const PrivateTag &csa2 = CSAHeader::GetCSASeriesHeaderInfoTag();
       const PrivateTag mec_mr3(0x700d, 0x08, "TOSHIBA_MEC_MR3");
-      const PrivateTag pmtf1(0x0029, 0x90, "PMTF INFORMATION DATA");
-      const PrivateTag pmtf2(0x0029, 0x90, "TOSHIBA_MEC_MR3");
+      static const PrivateTag &pmtf1 =
+          gdcm::MEC_MR3::GetPMTFInformationDataTag();
+      static const PrivateTag &pmtf2 = gdcm::MEC_MR3::GetToshibaMECMR3Tag();
+      static const PrivateTag &pmtf3 = gdcm::MEC_MR3::GetCanonMECMR3Tag();
 
       if (pt == csa1) {
         const bool ret = CleanCSA(ds, de);
@@ -1054,6 +1088,9 @@ bool Cleaner::impl::ProcessDataSet(Subject &subject, File &file, DataSet &ds,
       } else if (pt == pmtf2) {
         const bool ret = CleanPMTF(ds, de);
         if (!ret) return false;
+      } else if (pt == pmtf3) {
+        const bool ret = CleanPMTF(ds, de);
+        if (!ret) return false;
       } else {
         gdcmErrorMacro(" not implemented");
         return false;
